const readline = require('readline');
const read = require('fs-readdir-recursive');
const fs = require('fs');

const reader = readline.createInterface({
  input: process.stdin,
  output: process.stdout,
});

console.log(
  `\nInnan vi forsätter med uppgraderingen till version 21.0.X, måste du se till
    - att du står i rotmappen av projektet du vill uppgradera.
    - att du inte har några ändringar som inte är committade i din branch.

  Obs: Detta script endast går igenom HTML-filerna i ditt projekt,
       finns det ändra typer av filer med styleguide kod som t.ex. ts, tsx, js,
       less, scss etc. så får ni manuellt justera dessa.`
);

reader.question('\nVill du fortsätta? (y/n)', async function (svar) {
  if (svar.toLowerCase() == 'y') {
    console.log('Letar igenom alla html filer.\n');
    const htmlFiles = read(process.cwd())
      .filter(item => item.endsWith('.html'))
      .filter(item => !item.startsWith('node_modules'))
      .filter(item => !item.startsWith('bin'))
      .filter(item => !item.startsWith('dist'));

    let filesWithExternHeader = [];
    let filesWithHorisontallRadioBtn = [];
    let filesWithErrorClass = [];
    for (const htmlFile of htmlFiles) {
      lookingForOldHeadersToUpdate(htmlFile);
      lookingForExternalHeadersToUpdate(htmlFile, filesWithExternHeader);
      lookingForHorizontalRadioBtn(htmlFile, filesWithHorisontallRadioBtn);
      lookingForFilesWithErrorClass(htmlFile, filesWithErrorClass);
      lookingForGammalMultitext(htmlFile);
      lookingForGammalMultiSelect(htmlFile);
      lookingForButtons(htmlFile);
      lookingForPrimaryButtons(htmlFile);
      lookingForFilesWithPrimaryDarkButton(htmlFile);
      lookingForAnchorLinks(htmlFile);
      lookingForSektionsmeddelandeToUpdate(htmlFile);
    }

    console.log(`
    \nManuella åtgärder:
      1. "Sidhuvud för e-tjänster" har ny HTML struktur, gå in på https://styleguide.jordbruksverket.se/komponenter/sidhuvud.html och kopiera HTML koden.
      2. Vi har lagt in så att men på jv.fieldset behöver lägga in om innehållet ska visas horisontellt, du behöver lägga in attributet 'align="horizontal"' på jv-fieldset om du vill att radioknappar ska visas horisontellt.
      3. css only inmatningsfält / select > feltext(.error) > kryss-ikonen ska ha en wrapperklass (.error__iconwrapper) runt ikonen.
      4. Knappar har ny css klass struktur. c-btn är huvud klassen med 5 modifiers, c-btn--small,
         c-btn--compact, c-btn--primary, c-btn--dark. c-primary-btn, c-btn-primary--dark kommer inte
         fungera längre. Vi kör en automatisk uppdatering på endast html filer.
      5.Länkar har ny klass struktur. c-link är huvud klassen med 2 modifiers, c-link--dark och c-link--icon-right.
        Vi kör en automatisk uppdatering på endast html filer.
      `);
    if (filesWithExternHeader.length > 0) {
      console.log(`\nHittade manuella åtgärd nr.1: Du behöver uppdatera följande filer med gammal externa sidhuvud:`);
      for (const file of filesWithExternHeader) {
        console.log('* ' + file);
      }
    }
    if (filesWithHorisontallRadioBtn.length > 0) {
      console.log(
        `\nHittade manuella åtgärd nr.2: Vi har lagt in så att men på jv.fieldset behöver lägga in om innehållet ska visas horisontellt, du behöver lägga in attributet 'align="horizontal"' på jv-fieldset om du vill att radioknappar ska visas horisontellt`
      );
      for (const file of filesWithHorisontallRadioBtn) {
        console.log('* ' + file);
      }
    }
    if (filesWithErrorClass.length) {
      console.log(
        `\nHittade manuella åtgärd nr.3: Dubbelkolla alla .error klasser kryss-ikonen har en wrapper (.error__iconwrapper) :`
      );
      for (const file of filesWithErrorClass) {
        console.log('* ' + file);
      }
    }
    await reader.close();
  } else {
    await reader.close();
  }
});

const lookingForOldHeadersToUpdate = function (htmlFile) {
  let data = fs.readFileSync(htmlFile, 'utf8');

  // Uppdaterar c-header--black till c-header, som är till etjänster
  if (/\"c-header--black\"/i.test(data)) {
    data = data.replace(/\"c-header--black\"/g, `"c-header"`);
    writeToFile(htmlFile, data, '\nHittat c-header--black och ändrat till c-header c-header--black i ' + htmlFile);
  }
  // Uppdaterar c-header--black__content till c-header__content
  if (/c-header--black__content/i.test(data)) {
    data = data.replace(/--black__content/g, `__content`);
    writeToFile(htmlFile, data, '\nHittat c-header--black__content och ändrat till c-header__content i ' + htmlFile);
  }
  // Uppdaterar c-header--large till c-header c-header--large
  if (/\"c-header--large\"/i.test(data)) {
    data = data.replace(/\"c-header--large\"/g, `"c-header c-header--large"`);
    writeToFile(htmlFile, data, '\nHittat c-header--large och ändrat till c-header c-header--large i ' + htmlFile);
  }
  // Uppdaterar c-header--large__content till c-header__content
  if (/c-header--large__content/i.test(data)) {
    data = data.replace(/--large__content/g, `__content`);
    writeToFile(htmlFile, data, '\nHittat c-header--large__content och ändrat till c-header__content i ' + htmlFile);
  }

  // --title ska vara __title
  if (/c-header__content--title/i.test(data)) {
    data = data.replace(/--title/g, `__title`);
    writeToFile(
      htmlFile,
      data,
      '\nHittat c-header__content--title och ändrat till c-header__content__title i ' + htmlFile
    );
  }
};

const lookingForSektionsmeddelandeToUpdate = function (htmlFile) {
  let data = fs.readFileSync(htmlFile, 'utf8');
  let nbrOfString = 0;
  if (/<jv-section-banner.*role="/i.test(data)) {
    ({ data, nbrOfString } = findAndUpdateSektionsmeddelande(data, nbrOfString, /<jv-section-banner.*role="/gm));
    if (nbrOfString > 0) {
      writeToFile(
        htmlFile,
        data,
        '\nHittat och ändrat attribut "role" till "role-attr" på ' + nbrOfString + 'st sektionsmeddelande i, ' + htmlFile
      );
    }
  }
};

const lookingForExternalHeadersToUpdate = function (htmlFile, filesWithExternHeader) {
  let data = fs.readFileSync(htmlFile, 'utf8');
  if (/c-header\s+c-header--large/i.test(data)) {
    filesWithExternHeader.push(htmlFile);
  }
};

const lookingForHorizontalRadioBtn = function (htmlFile, filesWithHorisontallRadioBtn) {
  let data = fs.readFileSync(htmlFile, 'utf8');
  if (/align="horizontal"/i.test(data)) {
    filesWithHorisontallRadioBtn.push(htmlFile);
  }
};

const lookingForFilesWithErrorClass = function (htmlFile, filesWithErrorClass) {
  let data = fs.readFileSync(htmlFile, 'utf8');
  const pattern = /class\s*=\s*"(?:[^"]*\s)?\berror\b(?:\s[^"]*)?"/i;
  if (pattern.test(data)) {
    filesWithErrorClass.push(htmlFile);
  }
};

const lookingForGammalMultitext = function (htmlFile) {
  let data = fs.readFileSync(htmlFile, 'utf8');
  // Uppdaterar jv-multitext till jv-multitext-v1
  if (/jv-multitext(?!-v1)/i.test(data)) {
    data = data.replace(/<jv-multitext(?!-v1)/g, `<jv-multitext-v1`);
    writeToFile(
      htmlFile,
      data,
      '\nHittad en multitext ändra till multitext-v1 så att den gamla designen behålls i ' + htmlFile
    );
  }
};

const lookingForGammalMultiSelect = function (htmlFile) {
  let data = fs.readFileSync(htmlFile, 'utf8');
  // Uppdaterar jv-multi-select till jv-multi-select-v1
  if (/jv-multi-select(?!-v1)/i.test(data)) {
    data = data.replace(/<jv-multi-select(?!-v1)/g, `<jv-multi-select-v1`);
    writeToFile(
      htmlFile,
      data,
      '\nHittad en multi-select ändra till multi-select-v1 så att den gamla designen behålls i ' + htmlFile
    );
  }
};

const lookingForButtons = function (htmlFile) {
  let data = fs.readFileSync(htmlFile, 'utf8');
  if (/c-btn--(medium|compact|small)/i.test(data)) {
    data = data.replace(/c-btn--(medium|compact|small)/gi, match => {
      switch (match.toLowerCase()) {
        case 'c-btn--medium':
          return 'c-btn';
        case 'c-btn--compact':
          return 'c-btn c-btn--compact';
        case 'c-btn--small':
          return 'c-btn c-btn--small';
        default:
          return match;
      }
    });

    writeToFile(htmlFile, data, '\nHittat c-btn--(medium|compact|small) och ändrat i ' + htmlFile);
  }
};

const lookingForPrimaryButtons = function (htmlFile) {
  let data = fs.readFileSync(htmlFile, 'utf8');
  if (/c-primary-btn--(medium|compact|small)/i.test(data)) {
    data = data.replace(/c-primary-btn--(medium|compact|small)/gi, match => {
      switch (match.toLowerCase()) {
        case 'c-primary-btn--medium':
          return 'c-btn c-btn--primary';
        case 'c-primary-btn--compact':
          return 'c-btn c-btn--primary c-btn--compact';
        case 'c-primary-btn--small':
          return 'c-btn c-btn--primary c-btn--small';
        default:
          return match;
      }
    });

    writeToFile(htmlFile, data, '\nHittat c-primary-btn--(medium|compact|small) och ändrat i ' + htmlFile);
  }
};

const lookingForFilesWithPrimaryDarkButton = function (htmlFile, filesWithErrorClass) {
  let data = fs.readFileSync(htmlFile, 'utf8');
  if (/c-btn-primary--dark/i.test(data)) {
    data = data.replace(/c-btn-primary--dark/gi, `c-btn--dark`);
    writeToFile(htmlFile, data, '\nHittat c-btn-primary--dark och ändrat till c-btn--dark i ' + htmlFile);
  }
};

const lookingForAnchorLinks = function (htmlFile) {
  let data = fs.readFileSync(htmlFile, 'utf8');
  const replacementMap = {
    'c-link__light': 'c-link',
    'c-link__dark': 'c-link c-link--dark',
    'c-link__light--icon-right': 'c-link c-link--icon-right',
    'c-link__dark--icon-right': 'c-link c-link--dark c-link--icon-right',
  };

  data = data.replace(
    /c-link__light(?!--)|c-link__dark(?!--)|c-link__light--icon-right|c-link__dark--icon-right/gi,
    match => {
      return replacementMap[match.toLowerCase()] || match;
    }
  );

  writeToFile(
    htmlFile,
    data,
    '\nHittat (c-link__light|c-link__dark|c-link__light--icon-right|c-link__dark--icon-right) och ändrat i ' + htmlFile
  );
};

function findAndUpdateSektionsmeddelande(data, nbrOfString, regexp) {
  const matchingArray = data.match(regexp);

  if (!matchingArray) {
    return { data, nbrOfString };
  }
  for (const matchingString of matchingArray) {
    if (matchingString.includes('role="')) {
      data = replaceAll(
        data,
        matchingString,
        matchingString.replace(
          'role="',
          'role-attr="'
        )
      );
      nbrOfString = nbrOfString + 1;
    }
  }
  return { data, nbrOfString };
}

function writeToFile(htmlFile, data, message) {
  fs.writeFileSync(htmlFile, data, 'utf8');
  console.log(message);
}

reader.on('close', function () {
  console.log('\nAvslutar!');
  process.exit(0);
});

function replaceAll(string, match, replaceString) {
  return string.split(match).join(replaceString);
}
